import { Injectable } from "@angular/core";
import { Observable } from "rxjs/Observable";
import { Observer } from "rxjs/Observer";
import PouchDB from "pouchdb";

import * as PouchFind from "pouchdb-find";

import { UUID } from "angular2-uuid";

import "rxjs/add/observable/throw";
import "rxjs/add/observable/fromPromise";
import "rxjs/add/observable/from";
import "rxjs/add/operator/concatMap";

PouchDB.plugin(PouchFind);

export enum SaveAction {
    Add = 1,
    Update = 2,
    Delete = 3
}

@Injectable()
export class PouchService {

    pouchDB: any;

    constructor() {
        this.initialize();
    }

    /**
     *
     */
    private initialize(destroy?: boolean): Observable<any> {
        this.pouchDB = new PouchDB("odoo", {
            auto_compaction: true
        });

        return Observable.fromPromise(this.pouchDB.createIndex({
            index: {
                fields: ['odoo_model']
            }
        }));
    }


    /**
     *
     */
    save(data: any, action?: SaveAction): Observable<any> {
        if (!data.odoo_model) {
            return Observable.throw("Error: Cannot save data without model name");
        }

        return this.getAll(data.odoo_model).concatMap(result => this.pouchDB.post(result.docs.length !== 0 ? Object.assign(result.docs.shift(), { records: data.records }) : data ));
        // return this.getAll(data.odoo_model).concatMap(r => this.pouchDB.post(this.alignData(r.docs.shift(), data, action)));
    }

    /**
     * 
     * @param dataStored 
     * @param newData 
     * @param action 
     */
    protected alignData(dataStored: any, data: any, action?: SaveAction) {
        if (!dataStored) {
            return this.fillRecordsWithUUID(data);
        }

        if (!action) {
            dataStored.records = data.records;
            return this.fillRecordsWithUUID(dataStored);
        }

        if (action === SaveAction.Add) {
            dataStored.records = this.fillRecordsWithUUID(data).records.concat(dataStored.records);
            return dataStored;
        }

        if (action === SaveAction.Update) {

        }

        if (action === SaveAction.Delete) {

        }
    }

    /**
     * 
     * @param data 
     */
    protected fillRecordsWithUUID(data: any): any {
        data.records = data.records.map(record => {
            record.odoo_id = record.id;
            record.id = UUID.UUID();

            return record;
        });

        return data;
    }

    /**
     *
     */
    remove(data: any): Observable<any> {
        if (!data._id || !data._rev) {
            return Observable.throw("Error: Cannot remove data without id or rev");
        }

        return Observable.fromPromise(this.pouchDB.remove(data));
    }

    /**
     *
     */
    removeAll(type: string): Observable<any> {
        return this.getAll(type).concatMap(result => this.cleanDoc(result.docs.shift()));
    }

    /**
     * 
     * @param doc 
     */
    private cleanDoc(doc: any): Observable<any> {
        if (!doc || !doc.records) {
            return Observable.empty();
        }

        doc.records = [];
        
        return this.save(doc);
    }

    /**
     *
     */
    get(data: any): Observable<any> {
        if (!data._id) {
            return Observable.throw("Error: Cannot get data without id");
        }

        return Observable.fromPromise(this.pouchDB.get(data._id));
    }

    /**
     *
     */
    getAll(odoo_model: string): Observable<any> {
        return Observable.fromPromise(this.pouchDB.find({
            selector: {
                odoo_model: {
                    $eq: odoo_model
                }
            }
        }));
    }

    /**
     *
     */
    destroy(): Observable<any> {
        return Observable.fromPromise(this.pouchDB.destroy());
    }

    /**
     *
     */
    destroyAndInitialize() {
        return this.destroy().concatMap(response => this.initialize(response.ok));
    }
}
